/*
PIO1-SM0:
Ausgabe eines Bytes an den aktivierten PIO-Ausgaengen 15 - 22.
Ein einziger Befehl wartet auf neus Byte im FIFO und gibt es sofort aus.

PIO0-SM0:
Empfaengt Daten mit 500 kB im Format 8N1 mit 16-fach Abtastung und trigger
per IRQ1 das Reset-Monoflop. 

PIO0-SM1:
nachtriggerbares Monoflop 100 mS, was per IRQ1 (neues Byte empfangen) oder 
per Einsprung in die Routine ausgeloest wird: trigger_reset().

2025-04-21
Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/

#include "rom_simul.h"

extern uint32_t SystemCoreClock;
   
#define D0_BASIS    15      // ab hier Datenausgaenge
#define RESET_PIN   28      // Ausgang von Reset-Monoflop
#define RX_PIN      27      // RX-Eingang per PIO0-SM0


void init_PIO1(void)
{
// enable PIO1 
  SIO->GPIO_OE_SET = 0xff << D0_BASIS;  // Datenleitungen ab GPIO15 auf Ausgang  
  RESETS_CLR->RESET = RESETS_RESET_pio1_Msk;
  while (!(RESETS->RESET_DONE & RESETS_RESET_pio1_Msk));
  
// enable GPIOs
  while((RESETS->RESET_DONE & (RESETS_RESET_io_bank0_Msk | RESETS_RESET_pads_bank0_Msk)) !=
        (RESETS_RESET_io_bank0_Msk | RESETS_RESET_pads_bank0_Msk));

// 8 x Ausgaenge
  IO_BANK0->GPIO15_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_pio1_0;
  IO_BANK0->GPIO16_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_pio1_0;
  IO_BANK0->GPIO17_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_pio1_0;
  IO_BANK0->GPIO18_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_pio1_0;
  IO_BANK0->GPIO19_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_pio1_0;
  IO_BANK0->GPIO20_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_pio1_0;
  IO_BANK0->GPIO21_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_pio1_0;
  IO_BANK0->GPIO22_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_pio1_0;

  
// set output group D0 - D3 starting @GPIO15
  PIO1->SM0_PINCTRL =  4<<PIO0_SM0_PINCTRL_SET_COUNT_Pos | D0_BASIS<<PIO0_SM0_PINCTRL_SET_BASE_Pos; // |
  PIO1->SM0_INSTR = 0xe08f;                   // set pindirs D0 - D3
// set output group D4 - D5 starting @GPIO19 
  PIO1->SM0_PINCTRL =  4<<PIO0_SM0_PINCTRL_SET_COUNT_Pos | (D0_BASIS+4)<<PIO0_SM0_PINCTRL_SET_BASE_Pos; // |
  PIO1->SM0_INSTR = 0xe08f;                   // set pindirs D4 - D7
  PIO1->SM0_PINCTRL =  D0_BASIS << PIO0_SM0_PINCTRL_OUT_BASE_Pos | 
                        8 << PIO0_SM0_PINCTRL_OUT_COUNT_Pos;      // OUT-cmd: D0 - D7

  // init transfer mode
  PIO1->SM0_SHIFTCTRL |= PIO0_SM0_SHIFTCTRL_AUTOPULL_Msk |        // autopull of DMA-data
                         8 << PIO0_SM0_SHIFTCTRL_PULL_THRESH_Pos; // 8 bit
// der einzige Befehl in Dauerschleife
  PIO1->INSTR_MEM0 = 0x6008;                          // OUT 8 bits
  
  PIO1->SM0_INSTR = 0x0000;                           // jmp SM0 start
  PIO1->SM0_EXECCTRL = (0 << PIO0_SM0_EXECCTRL_WRAP_TOP_Pos) | 
                       (0 << PIO0_SM0_EXECCTRL_WRAP_BOTTOM_Pos);  
 
  PIO1->SM0_CLKDIV = 1<<PIO0_SM0_CLKDIV_INT_Pos;      // max. speed
  PIO1_SET->CTRL = (1 << (PIO0_CTRL_SM_ENABLE_Pos));  // start
}


void init_PIO0_RX(void)
{
  uint32_t *prg, baud_temp;
  RESETS_CLR->RESET = RESETS_RESET_pio0_Msk;
  while (!(RESETS->RESET_DONE & RESETS_RESET_pio0_Msk));
  
// enable GPIOs
  while((RESETS->RESET_DONE & (RESETS_RESET_io_bank0_Msk | RESETS_RESET_pads_bank0_Msk)) !=
        (RESETS_RESET_io_bank0_Msk | RESETS_RESET_pads_bank0_Msk));

  PIO0->SM0_PINCTRL =  RX_PIN << PIO0_SM0_PINCTRL_IN_BASE_Pos;       // nur 1 x
  IO_BANK0->GPIO27_CTRL |= 1 << IO_BANK0_GPIO0_CTRL_INOVER_Pos;       // invertierter Eingang
  PIO0->SM0_EXECCTRL |=  (RX_PIN  << PIO0_SM0_EXECCTRL_JMP_PIN_Pos);  // Pin fuer Startbit Test

  PIO0->SM0_SHIFTCTRL |= PIO0_SM0_SHIFTCTRL_FJOIN_RX_Msk;             // 8 x RX-fifo
  
#ifdef DIVIDER_8  
  
  baud_temp = SystemCoreClock / DEF_BAUDRATE / 8;
  prg = (uint32_t *)&PIO0->INSTR_MEM0;    // pgm_start
// ADR0 
  *prg++ = 0x2300 + RX_PIN;     // WAIT auf '0' @ RX_PIN + 3 x wait
// ADR1 
  *prg++ = 0x00c0;              // JMP pin Startbit nach 4 Takten testen
// ADR2 
  *prg++ = 0xe620 + 7;          // SET x,7 6 x wait; 1. Bit nach 12 Takten lesen!
// ADR3 
  *prg++ = 0x4001;              // IN pins, RX-bit
// ADR4 
  *prg++ = 0x0643;              // JMP x--, ADR3, 6 x wait: insgesamt 8 Takte / Bit
// ADR5 
  *prg++ = 0x00c7;              // JMP pin gueltiges Stoppbit: Wert ausgeben
// ADR6 
  *prg++ = 0x0000;              // JMP pin Stoppbit fehlt
// ADR7
  *prg++ = 0xc001;              // IRQ, irq1-flag setzen; Reset triggern
// ADR8
  *prg++ = 0x8000;              // PUSH   Wert ausgeben
// ADR9
  *prg++ = 0x0000 ;             // und wieder auf Anfang
 
#else 

  baud_temp = SystemCoreClock / DEF_BAUDRATE / 16;
  prg = (uint32_t *)&PIO0->INSTR_MEM0;    // pgm_start
// ADR0 
  *prg++ = 0x2700 + RX_PIN;     // WAIT auf '0' @ RX_PIN + 3 x wait
// ADR1 
  *prg++ = 0x00c0;              // JMP pin Startbit nach 8 Takten testen
// ADR2 
  *prg++ = 0xef20 + 7;          // SET x,7 15 x wait; 1. Bit nach 24 Takten lesen!
// ADR3 
  *prg++ = 0x4001;              // IN pins, RX-bit
// ADR4 
  *prg++ = 0x0e43;              // JMP x--, ADR3, 6 x wait: insgesamt 16 Takte / Bit
// ADR5 
  *prg++ = 0x00c7;              // JMP pin gueltiges Stoppbit: Wert ausgeben
// ADR6 
  *prg++ = 0x0000;              // JMP pin Stoppbit fehlt
// ADR7
  *prg++ = 0xc001;              // IRQ, irq1-flag setzen; Reset triggern
// ADR8
  *prg++ = 0x8000;              // PUSH   Wert ausgeben
// ADR9
  *prg++ = 0x0000 ;             // und wieder auf Anfang
#endif
  
  PIO0->SM0_INSTR = 0x0000;         // jmp SM0 start
  PIO0->SM0_EXECCTRL =  (RX_PIN  << PIO0_SM0_EXECCTRL_JMP_PIN_Pos) |
                        (8 << PIO0_SM0_EXECCTRL_WRAP_TOP_Pos) | 
                        (0 << PIO0_SM0_EXECCTRL_WRAP_BOTTOM_Pos);  

  PIO0->SM0_CLKDIV = baud_temp << PIO0_SM0_CLKDIV_INT_Pos;
  PIO0_SET->CTRL = (1 << (PIO0_CTRL_SM_ENABLE_Pos));  // start

  PIO0->IRQ0_INTE |= PIO0_INTR_SM0_RXNEMPTY_Msk;
  NVIC_EnableIRQ(PIO0_IRQ_0_IRQn);
// RX ist fertig  

// fuer Reset-Ausgang nachtriggerbar
  IO_BANK0->GPIO28_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_pio0_0;
  PIO0->SM1_PINCTRL =  1<<PIO0_SM0_PINCTRL_SET_COUNT_Pos | RESET_PIN<<PIO0_SM0_PINCTRL_SET_BASE_Pos; // |
  PIO0->SM1_INSTR = 0xe081;      // set pindirs
 
  prg = (uint32_t *)&PIO0->INSTR_MEM10;
// ADR10 
  *prg++ = 0xe000;              // SET RESET_PIN = 0
// ADR11 
  *prg++ = 0x20c1;              // WAIT auf '1' @ IRQ1
// ADR12 
  *prg++ = 0xe001;              // SET RESET_PIN = 1
// ADR13 
  *prg++ = 0xc041;              // IRQ 1 = 0
// ADR14 
  *prg++ = 0xa027;              // MOV X,OSR
// ADR15 
  *prg++ = 0x0040 + 15;         // JMP x--, ADR15, Warteschleife mit x-reg
  *prg++ = 0x0000 + 10;         // JMP ADR10 zum Anfang

  PIO0->TXF1 = SystemCoreClock/10;                    // 100 ms
  PIO0->SM1_INSTR = 0x8080;                           // PULL OSR
  PIO0->SM1_INSTR = 0x0000 + 12;                      // mit Reset-impuls starten
  PIO1->SM0_CLKDIV = 1<<PIO0_SM0_CLKDIV_INT_Pos;      // max. speed
  PIO0_SET->CTRL = (2 << (PIO0_CTRL_SM_ENABLE_Pos));  // start
  
}

void trigger_reset(void)
{
  PIO0->SM1_INSTR = 0x0000 + 12;     // Reset ausloesen
}
