/* 
Zum Abspeichern der Daten in den FLASH-Speicher mu der XIP-Zugriff 
gesperrt werden. Die Daten werden anschliessend ins FLASH geschrieben und der
XIP-Zugriff wieder aktiviert.

2025-04-21
Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/
#include "rom_simul.h"

extern uint8_t rom_puffer[];
extern void trigger_reset(void);


#define W25_BUSY_BIT        1
#define W25_WEL_BIT         2

#define W25_PAGE_PROGRAMM   2
#define W25_READ_DATA       3
#define W25_READ_STAT1      5
#define W25_WR_ENABLE       6
#define W25_WR_DISABLE      4
#define W25_SEC_ERASE       0x20
#define W25_BLK_ERASE       0x52
#define W25_MANU_ID         0x90
#define W25_POWER_UP        0xab
#define W25_POWER_DOWN      0xb9

uint32_t W25_status, SPI1_temp;

void SPI1_start(void)
{
//  XIP_SSI->SSIENR = 1;
    IO_QSPI->GPIO_QSPI_SS_CTRL = IO_QSPI->GPIO_QSPI_SS_CTRL & ~IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_Msk
        | (2 << IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_Pos);
}


void SPI1_FIFO_leeren(void)
{
  while((XIP_SSI->SR & XIP_SSI_SR_BUSY_Msk));     // SPI Aktivitaeten abwarten
  while((XIP_SSI->SR & XIP_SSI_SR_RFNE_Msk))      // und FIFO leeren
    *(__IO uint8_t *)&XIP_SSI->DR0;
}

void SPI1_stop(void)
{
  SPI1_FIFO_leeren();                             // warten bis Ausgabe erledigt ist
  IO_QSPI->GPIO_QSPI_SS_CTRL = IO_QSPI->GPIO_QSPI_SS_CTRL & ~IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_Msk
        | (3 << IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_Pos);
  IO_QSPI->GPIO_QSPI_SS_CTRL = IO_QSPI->GPIO_QSPI_SS_CTRL & ~IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_Msk
        | (0 << IO_QSPI_GPIO_QSPI_SS_CTRL_OUTOVER_Pos);
} 

void SPI1_write(uint8_t cmd)
{
  *(__IO uint8_t *)&XIP_SSI->DR0 = (uint8_t)(cmd);  // Wert als Byte schreiben
    while(!(XIP_SSI->SR & XIP_SSI_SR_TFE_Msk));     // Ausgabe abwarten
}


// WERT holen
uint8_t SPI1_read(void)
{
 uint8_t data;
  *(__IO uint8_t *)&XIP_SSI->DR0 = 0;             // nur Taktsignal erzeugen
  while(!(XIP_SSI->SR & XIP_SSI_SR_RFNE_Msk));    // auf Empfang warten
  data = *(__IO uint8_t *)&XIP_SSI->DR0;          // als Byte lesen
  return(data);
}

void init_SPI1(void)
{
  RESETS_CLR->RESET = RESETS_RESET_spi1_Msk;
  while (!(RESETS->RESET_DONE & RESETS_RESET_spi1_Msk));
  XIP_SSI->SSIENR = 0;
  (void) XIP_SSI->SR;
  (void) XIP_SSI->ICR;

  XIP_SSI->BAUDR = 6;
  XIP_SSI->CTRLR0 =
          (XIP_SSI_CTRLR0_SPI_FRF_STD << XIP_SSI_CTRLR0_SPI_FRF_Pos) |  // Standard 1-bit SPI serial frames
          (7 << XIP_SSI_CTRLR0_DFS_32_Pos) |                            // 8 clocks per data frame
          (XIP_SSI_CTRLR0_TMOD_TX_AND_RX << XIP_SSI_CTRLR0_TMOD_Pos);   // TX and RX FIFOs are both used for every byte
  XIP_SSI->SER = 1;
  XIP_SSI->SSIENR = 1;
}

 
void W25_adr24(uint32_t adr)
{
  SPI1_write(adr>>16);
  SPI1_write(adr>>8);
  SPI1_write(adr);
  SPI1_FIFO_leeren();                             // empfangene Daten ignorieren und loeschen
}  
  
void W25_write_enable(void)
{
  SPI1_start();
  SPI1_write(W25_WR_ENABLE);
  SPI1_stop();
}

void W25_write_disable(void)
{
  SPI1_start();
  SPI1_write(W25_WR_DISABLE);
  SPI1_stop();
}  

uint32_t W25_read_status1(void)
{
uint32_t temp;  
  SPI1_start();
  SPI1_write(W25_READ_STAT1);
  SPI1_FIFO_leeren();

  temp = SPI1_read();
  SPI1_stop();
  return(temp);
}  

uint32_t W25_busy(void)
{
  return(W25_read_status1() & W25_BUSY_BIT);
}

void W25_power_up(void)
{
  SPI1_start();
  SPI1_write(W25_POWER_UP);
  SPI1_stop();
} 

void W25_power_down(void)
{
  SPI1_start();
  SPI1_write(W25_POWER_DOWN);
  SPI1_stop();
} 


void W25_sec_erase(uint32_t adr)
{
  W25_write_enable();  
  SPI1_start();
  SPI1_write(W25_SEC_ERASE);
  W25_adr24(adr);
  SPI1_stop();
}  
  
uint32_t W25_read_manu_id(void)
{
uint32_t temp; 
  SPI1_FIFO_leeren();
  SPI1_start();
  SPI1_write(W25_MANU_ID);
  W25_adr24(0);                                   // dummy Adresse + FIFO leeren 
  temp = SPI1_read() << 8;
  temp += SPI1_read();  
  SPI1_stop();
  return(temp);
}         

void W25_init_write(uint32_t adr)
{
  SPI1_start();
  SPI1_write(W25_WR_ENABLE);
  SPI1_stop();
  SPI1_start();
  SPI1_write(W25_PAGE_PROGRAMM);
  W25_adr24(adr);
}

void W25_init_read(uint32_t adr)
{
  SPI1_start();
  SPI1_write(W25_READ_DATA);
  W25_adr24(adr);

}

void init_xip(void)
{
  XIP_SSI->SSIENR = 0;
  // Setup for XIP
  XIP_SSI->CTRLR0 = (XIP_SSI_CTRLR0_SPI_FRF_STD << XIP_SSI_CTRLR0_SPI_FRF_Pos) |
      (XIP_SSI_CTRLR0_TMOD_EEPROM_READ << XIP_SSI_CTRLR0_TMOD_Pos) |
      ((32-1) << XIP_SSI_CTRLR0_DFS_32_Pos);

  XIP_SSI->CTRLR1 = (0 << XIP_SSI_CTRLR1_NDF_Pos);

  XIP_SSI->SPI_CTRLR0 = (0x03 << XIP_SSI_SPI_CTRLR0_XIP_CMD_Pos) |  // read data cmd
    ((24 / 4) << XIP_SSI_SPI_CTRLR0_ADDR_L_Pos) |
    (XIP_SSI_SPI_CTRLR0_INST_L_8B << XIP_SSI_SPI_CTRLR0_INST_L_Pos) |
    (XIP_SSI_SPI_CTRLR0_TRANS_TYPE_1C1A << XIP_SSI_SPI_CTRLR0_TRANS_TYPE_Pos);

  XIP_SSI->SSIENR = XIP_SSI_SSIENR_SSI_EN_Msk; 
}


void speicher_rom_puffer(uint32_t offset)
{
  uint32_t i, j, adr;
  adr = ROM_DATEN_START + offset;
  init_SPI1();
  for(i = 0; i < MAX_ROM_SIZE; ) {                // i++ weiter unten !
    if((adr & 0xfff) == 0) {
      W25_sec_erase(adr);  
      trigger_reset();
      while(W25_busy());
    }
    W25_init_write(adr);                          // Zugriff freigeben
    for(j = 0; j < 256; j++) {                    // byteweise ausgeben
      SPI1_write(rom_puffer[i++]);
    }
    SPI1_stop();  
    adr += 256;
    trigger_reset();
    while(W25_busy());
  }  
  init_xip();
}
